from main import Main
from vitamin import Vitamin
from mineral import Mineral
from other import Other

__version__ = "1.0.4"
__sources__ = [
    "https://www.fda.gov/files/food/published/Food-Labeling-Guide-%28PDF%29.pdf",
    "https://www.usdairy.com/~/media/usd/public/dmi-quick-reference-guide_nutrition-claims-for-dairy-products_2018.pdf",
]



from .utilities import vmo


class Vitamin:
    def vitamin_c(self, quantity: "int/str"):
        return vmo(quantity, 1)

    def vitamin_e(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def thiamine(self, quantity: "int/str"):  # Vitamin B12
        return vmo(quantity, 0.1)

    def riboflavin(self, quantity: "int/str"):  # Vitamin B2
        return vmo(quantity, 0.1)

    def niacin(self, quantity: "int/str"):  # Vitamin B3
        return vmo(quantity, 0.1)

    def vitamin_b6(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def pantothenic_acid(self, quantity: "int/str"):  # Vitamin B5
        return vmo(quantity, 0.1)

    def vitamin_a(self, quantity: "int/str"):
        return vmo(quantity, 10)

    def folate(self, quantity: "int/str"):  # Vitamin B9
        return vmo(quantity, 5)

    def vitamin_k(self, quantity: "int/str"):
        return vmo(quantity, 1)

    def biotin(self, quantity: "int/str"):  # Vitamin B7
        return vmo(quantity, 1)

    def vitamin_d(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def vitamin_b12(self, quantity: "int/str"):
        return vmo(quantity, 10)



from .utilities import vmo


class Mineral:
    def calcium(self, quantity: "int/str"):
        return vmo(quantity, 10)

    def potassium(self, quantity: "int/str"):
        return vmo(quantity, 10)

    def phosphorus(self, quantity: "int/str"):
        return vmo(quantity, 10)

    def magnesium(self, quantity: "int/str"):
        return vmo(quantity, 5)

    def iron(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def zinc(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def copper(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def manganese(self, quantity: "int/str"):
        return vmo(quantity, 0.1)

    def iodine(self, quantity: "int/str"):
        return vmo(quantity, 1)

    def selenium(self, quantity: "int/str"):
        return vmo(quantity, 1)

    def chromium(self, quantity: "int/str"):
        return vmo(quantity, 1)

    def molybdenum(self, quantity: "int/str"):
        return vmo(quantity, 1)



from .utilities import round_increment, parse_quantity


class Main:
    def calories(self, quantity: "int/str") -> str:
        value, unit = parse_quantity(quantity)
        if value is None or value < 5:
            return f"0{unit}"
        elif value <= 50:
            return f"{round_increment(value, 5)}{unit}"
        return f"{round_increment(value, 10)}{unit}"

    def __fat(self, quantity):
        value, unit = parse_quantity(quantity)
        unit = " g" if unit.strip() == "" else unit
        if value < 0.5:
            return f"0{unit}"
        elif value < 5:
            return f"{round_increment(value, 0.5)}{unit}"
        return f"{round_increment(value, 1)}{unit}"

    def tot_fat(self, quantity: "int/str") -> str:
        return self.__fat(quantity)

    def sat_fat(self, quantity: "int/str") -> str:
        return self.__fat(quantity)

    def trans_fat(self, quantity: "int/str") -> str:
        return self.__fat(quantity)

    def poly_fat(self, quantity: "int/str") -> str:
        return self.__fat(quantity)

    def mono_fat(self, quantity: "int/str") -> str:
        return self.__fat(quantity)

    def cholesterol(self, quantity: "int/str", minimal: bool = False) -> str:
        value, unit = parse_quantity(quantity)
        unit = " mg" if unit.strip() == "" else unit
        if value < 2:
            return f"0{unit}"
        elif value < 5:
            result = f"{round_increment(value, 1)}{unit}"
            return result if minimal else f"less than{result}"
        return f"{round_increment(value, 5)}{unit}"

    def __sp(self, quantity):
        value, unit = parse_quantity(quantity)
        unit = " mg" if unit.strip() == "" else unit
        if value < 5:
            return f"0{unit}"
        elif value < 140:
            return f"{round_increment(value, 5)}{unit}"
        return f"{round_increment(value, 10)}{unit}"

    def sodium(self, quantity: "int/str") -> str:
        return self.__sp(quantity)

    def potassium(self, quantity: "int/str") -> str:
        return self.__sp(quantity)

    def __carb(self, quantity, minimal):
        value, unit = parse_quantity(quantity)
        if value < 0.5:
            return f"0{unit}"
        elif value < 1:
            return f"<1{unit}" if minimal else f"less than 1{unit}"
        return f"{round_increment(value, 1)}{unit}"

    def tot_carb(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def dietary_fiber(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def soluble_fiber(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def insoluble_fiber(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def tot_sugars(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def added_sugars(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def sugar_alcohol(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def other_carb(self, quantity: "int/str", minimal: bool = False) -> str:
        return self.__carb(quantity, minimal)

    def protein(self, quantity: "int/str", minimal: bool = False) -> str:
        value, unit = parse_quantity(quantity)
        unit = " g" if unit.strip() == "" else unit
        if value < 0.5:
            return f"0{unit}"
        elif value < 1:
            return f"1{unit}" if minimal else f"less than 1{unit}"
        return f"{round_increment(value, 1)}{unit}"



from re import match


def parse_quantity(quantity):
    if quantity is None:
        return 0
    elif type(quantity) in [int, float]:
        return quantity
    quantity = quantity.strip()
    parsed = match(r"\d+(\.\d+)?", quantity).group()
    unit = quantity[len(parsed) :].strip() if not quantity.isdigit() else ""
    value = float(parsed) if "." in parsed else int(parsed)
    return value, f" {unit}"


def round_increment(value, increment):
    minimum = value // increment * increment
    maximum = minimum + increment
    res = maximum if (maximum - value) <= (value - minimum) else minimum
    return res if res % 1.0 else int(res)


# # https://stackoverflow.com/a/61212047/14767766
# def find_first_meaningful_decimal(x):
#   candidate = 0
#   MAX_DECIMAL = 10
#   EPSILON = 1 / 10 ** MAX_DECIMAL
#   while round(x, candidate) < EPSILON:
#       candidate += 1
#       if candidate > MAX_DECIMAL:
#           raise Exception('Number is too small: {}'.format(x))
#   if int(x * 10 ** (candidate + 1)) == 5:
#       candidate += 1
#   return candidate


# def round_increment(value, increment):
#   correction = 1e-5 if x > 0 else -1e-5
#   result = round(value / increment + correction) * increment
#   return round(result, find_first_meaningful_decimal(increment))


def vmo(quantity, increment):
    value, unit = parse_quantity(quantity)
    unit = " mcg" if unit.strip() == "" else unit
    return f"{round_increment(value, increment)}{unit}"
